<?php

namespace Estoque\Logic\TransferenciaEstoque;

use Estoque\Entities\Ctrlmoviestq;
use Estoque\Entities\Itlanctoest;
use Estoque\Entities\Lanctoestoq;
use Estoque\Logic\AtualizacaoEstoque\AtualizadorDeEstoque;
use Geral\Entities\Itrequesto;
use Geral\Entities\Requesto;
use Geral\Util\DateUtil;
use Geral\Util\TratamentoDeMensagens;

class TransferenciaEstoque {

    // Formata a data de forma que possa ser inserida no banco
    public static function formataData($data)
    {
        if (strlen($data) == 19)
            $data = DateUtil::truncateString($data);
        $data = DateUtil::getDataDeString($data);
        $data = DateUtil::truncateData($data);

        return $data;
    }

    public static function insertLancamentos(
        $filial, $numLancamento, $tipoLancamento, $dtLancamento, $descricao, $filialMovi,
        $operacao, $almoxarifado, $localizacao, $operador, $ativo, $dtInclusao,
        $operadorInclusao, $organizacao
    )
    {
        try {
            $connection = \Setforms2\Shared\Registry::getEntityManager();
            $connection->beginTransaction();
            /** @var Lanctoestoq $lancamento */
            $lancamento = new Lanctoestoq();
            $lancamento->setCdfilial($filial);
            $lancamento->setNrlancestq($numLancamento);
            $lancamento->setIdtplancto($tipoLancamento);
            $lancamento->setDtlancestq(self::formataData($dtLancamento));
            $lancamento->setDslancestq($descricao);
            $lancamento->setCdfilimovi($filialMovi);
            $lancamento->setIdentrsaidop($operacao);
            $lancamento->setCdalmoxarife($almoxarifado);
            $lancamento->setCdlocalestoq($localizacao);
            $lancamento->setIdstatusctbest('P');
            $lancamento->setCdoperador($operador);
            $lancamento->setCdoperinclusao($operadorInclusao);
            $lancamento->setCdoperultatu($operadorInclusao);
            $lancamento->setDtinclusao(self::formataData($dtInclusao));
            $lancamento->setDtultatu(self::formataData($dtInclusao));
            $lancamento->setNrorg($organizacao);
            $lancamento->setNrorginclusao($organizacao);
            $lancamento->setNrorgultatu($organizacao);
            $lancamento->persist();

            $connection->flush();
            $connection->commit();

        } catch (\Exception $e) {
            $connection->rollback();
            TratamentoDeMensagens::exibirMensagens($e, true);
        }
    }

    public static function deletaRequisicaoPorFilLancReqc($filial, $filialSaida, $lancamentoSaida, $organizacao)
    {
        try {
            $connection = \Setforms2\Shared\Registry::getEntityManager();
            $connection->beginTransaction();

            /** @var Requesto $requesto */
            $requesto = Requesto::findOneByCdfilialAndCdfillancreqcAndNrlancreqcAndNrorg(
                $filial, $filialSaida, $lancamentoSaida, $organizacao
            );

            if ($requesto)
                $requesto->delete();
            $connection->flush();
            $connection->commit();

        } catch (\Exception $e) {
            $connection->rollback();
            TratamentoDeMensagens::exibirMensagens($e, true);
        }
    }

    public static function updateSituacaoRequisicao($filial, $requisicao, $situacao, $filLancReqc, $lancReqc, $organizacao)
    {
        try {
            $connection = \Setforms2\Shared\Registry::getEntityManager();
            $connection->beginTransaction();

            /** @var Requesto $requesto */
            $requesto = Requesto::findOneByCdfilialAndNrrequestoAndNrorg($filial, $requisicao, $organizacao);
            $requesto->setCdfillancreqc($filLancReqc);
            $requesto->setNrlancreqc($lancReqc);
            $requesto->setIdstatusreq($situacao);
            $requesto->persist();

            $connection->flush();
            $connection->commit();
            \Setforms2\Shared\Registry::getEntityManager()->flush();

        } catch (\Exception $e) {
            $connection->rollback();
            TratamentoDeMensagens::exibirMensagens($e, true);
        }
    }

    public static function atualizaItemRequisicao($filial, $filialLanc, $lancamento, $produto, $requisicao, $seqItem)
    {
        try {
            $connection = \Setforms2\Shared\Registry::getEntityManager();
            $connection->beginTransaction();

            /** @var Itrequesto $itrequesto */
            $itrequesto = Itrequesto::findOneByCdfilialAndNrrequestoAndCdproduto($filial, $requisicao, $produto);
            $itrequesto->setCdfililanc($filialLanc);
            $itrequesto->setNrlancestq($lancamento);
            $itrequesto->setNrsequitem($seqItem);
            $itrequesto->persist();

            $connection->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollback();
            TratamentoDeMensagens::exibirMensagens($e, true);
        }
    }

    public static function updateItensSituacaoRequisicao(
        $filial, $filialLanc, $lancamento, $requisicao, $situacao, $organizacao
    )
    {
        try {
            $connection = \Setforms2\Shared\Registry::getEntityManager();
            $connection->beginTransaction();

            /** @var Itrequesto $item */
            $itensRequisicao = Itrequesto::findByCdfilialAndCdfililancAndNrlancestqAndNrrequestoAndNrorg(
                $filial, $filialLanc, $lancamento, $requisicao, $organizacao
            );
            foreach ($itensRequisicao as $item) {
                $item->setIdstitrequ($situacao);
                $item->setCdfililanc(NULL);
                $item->setNrlancestq(NULL);
                $item->setNrsequitem(NULL);
                $item->persist();
            }
            $connection->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollback();
            TratamentoDeMensagens::exibirMensagens($e, true);
        }
    }

    public static function insertItensLancamento(
        $filial, $lancamento, $seqItem, $produto, $quantLanc, $valorUnit, $valorTotalLanc,
        $tipoMovi, $dtLancMovi, $idOrdemClc, $valorLanctoEst, $qtTotalLancto, $prodEsto,
        $valorLanctoBrut, $idEvolEstoq, $idEntrSaidOp, $cdTipoOper, $descricao, $filialMovi,
        $almoxarifado, $localEstoque, $lote, $subLote, $operador, $qtDevolEstoq,
        $dtLancMoviAnt, $dtInclusao, $operadorInclusao, $organizacao
    )
    {
        try {
            $connection = \Setforms2\Shared\Registry::getEntityManager();
            $connection->beginTransaction();

            /** @var Itlanctoest $itlanctoest */
            $itlanctoest = new Itlanctoest();
            $itlanctoest->setCdfilial($filial);
            $itlanctoest->setNrlancestq($lancamento);
            $itlanctoest->setNrsequitem($seqItem);
            $itlanctoest->setDslancestit($descricao);
            $itlanctoest->setCdproduto($produto);
            $itlanctoest->setCdprodmovi($prodEsto);
            $itlanctoest->setQtlanctoest($quantLanc);
            $itlanctoest->setQttotlancto($qtTotalLancto);
            $itlanctoest->setQtdevolestoq($qtDevolEstoq);
            $itlanctoest->setVrunilancto($valorUnit);
            $itlanctoest->setVrtotlancto($valorTotalLanc);
            $itlanctoest->setVrlanctoest($valorLanctoEst);
            $itlanctoest->setVrlanctobrut($valorLanctoBrut);
            $itlanctoest->setDtlancmovi(self::formataData($dtLancMovi));
            $itlanctoest->setIdordemclc($idOrdemClc);
            $itlanctoest->setIdentrsaidop($idEntrSaidOp);
            $itlanctoest->setIdtipomovi($tipoMovi);
            $itlanctoest->setIdevolestoq($idEvolEstoq);
            $itlanctoest->setCdalmoxarife($almoxarifado);
            $itlanctoest->setCdlocalestoq($localEstoque);
            $itlanctoest->setNrloteestq($lote);
            $itlanctoest->setNrsublote($subLote);
            $itlanctoest->setCdoperador($operador);
            $itlanctoest->setCdtipooper($cdTipoOper);
            $itlanctoest->setCdfilimovi($filialMovi);
            $itlanctoest->setDtlancmovant(self::formataData($dtLancMoviAnt));
            $itlanctoest->setDtinclusao(self::formataData($dtInclusao));
            $itlanctoest->setDtultatu(self::formataData($dtInclusao));
            $itlanctoest->setCdoperinclusao($operadorInclusao);
            $itlanctoest->setCdoperultatu($operadorInclusao);
            $itlanctoest->setNrorg($organizacao);
            $itlanctoest->setNrorginclusao($organizacao);
            $itlanctoest->setNrorgultatu($organizacao);
            $itlanctoest->persist();

            $connection->flush();
            $connection->commit();

        } catch (\Exception $e) {
            $connection->rollback();
            TratamentoDeMensagens::exibirMensagens($e, true);
        }
    }

    public static function updateDataLancamento($filial, $lancamentoSaida, $lancamentoEntrada, $data, $organizacao)
    {
        try {
            $connection = \Setforms2\Shared\Registry::getEntityManager();
            $connection->beginTransaction();

            $where = 'CDFILIAL  = ?
                AND NRLANCESTQ  IN (?, ?)
                AND NRORG       = ?';

                $params = array(
                    $filial, $lancamentoSaida, $lancamentoEntrada, $organizacao
                );

                /** @var Itlanctoest $item */
                $itens = Itlanctoest::getObjectsByWhere($where, $params);
                foreach ($itens as $item) {
                    $item->setDtlancmovi(self::formataData($data));
                    $item->persist();
            }

            $connection->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollback();
            TratamentoDeMensagens::exibirMensagens($e, true);
        }
    }

    public static function deletaLancamentos($filial, $lancamentoEntrada, $lancamentoSaida, $organizacao)
    {
        try {

            $where = 'CDFILIAL  = ?
                AND NRLANCESTQ  IN (?, ?)
                AND NRORG       = ?';

            $params = array(
                $filial, $lancamentoSaida, $lancamentoEntrada, $organizacao
            );

            /** @var Lanctoestoq $lancamento */
            $lancamentos = Lanctoestoq::getObjectsByWhere($where, $params);
            foreach ($lancamentos as $lancamento) {
                $lancamento->delete();
            }
            \Setforms2\Shared\Registry::getEntityManager()->flush();

        } catch (\Exception $e) {
            TratamentoDeMensagens::exibirMensagens($e);
        }
    }

    public static function deletaItensLancamentos($filial, $lancamentoEntrada, $lancamentoSaida, $organizacao)
    {
        try {
            $where = 'CDFILIAL 	= ?
                AND NRLANCESTQ 	IN (?, ?)
                AND NRORG       = ?';

            $params = array(
                $filial, $lancamentoEntrada, $lancamentoSaida, $organizacao
            );

            /** @var Itlanctoest $item */
            $itens = Itlanctoest::getObjectsByWhere($where, $params);

            foreach ($itens as $item) {
                $item->delete();
            }
            \Setforms2\Shared\Registry::getEntityManager()->flush();

        } catch (\Exception $e) {
            TratamentoDeMensagens::exibirMensagens($e, true);
        }
    }

    public static function deletaControleMoviEstoque($filial, $numControle, $organizacao)
    {
        try {
            /** @var Ctrlmoviestq $ctrlmoviestq */
            $ctrlmoviestq = Ctrlmoviestq::findOneByCdfilialAndNrctrlmovestAndNrorg($filial, $numControle, $organizacao);
            $ctrlmoviestq->delete();

            \Setforms2\Shared\Registry::getEntityManager()->flush();

        } catch (\Exception $e) {
            TratamentoDeMensagens::exibirMensagens($e, true);
        }
    }

    public static function deletaTranferencia($filial, $lancamentoEntrada, $lancamentoSaida, $transferencia, $organizacao)
    {
        try {
            $connection = \Setforms2\Shared\Registry::getEntityManager();
            $connection->beginTransaction();

            self::deletaItensLancamentos($filial, $lancamentoEntrada, $lancamentoSaida, $organizacao);
            self::deletaControleMoviEstoque($filial, $transferencia, $organizacao);
            self::atualizaEstoqueLancamentos($filial, $lancamentoEntrada, $lancamentoSaida, $organizacao);
            self::deletaLancamentos($filial, $lancamentoEntrada, $lancamentoSaida, $organizacao);

            \Setforms2\Shared\Registry::getEntityManager()->flush();
            $connection->commit();

        } catch (\Exception $e) {
            $connection->rollback();
            TratamentoDeMensagens::exibirMensagens($e, true);
        }
    }

    public static function atualizaEstoqueLancamentos($filial, $lancamentoEntrada, $lancamentoSaida, $organizacao)
    {
        try {
            self::atualizaEstoque($filial, $lancamentoEntrada, $organizacao);
            self::atualizaEstoque($filial, $lancamentoSaida, $organizacao);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public static function atualizaEstoque($filial, $lancamento, $organizacao)
    {
        try {
            $connection = \Setforms2\Shared\Registry::getEntityManager();
            $connection->beginTransaction();
            /** @var $objLancto Lanctoestoq */
            $objLancto = Lanctoestoq::consultaLancamento($organizacao, $filial, $lancamento);

            /** @var $objItens Itlanctoest */
            $objItens = Itlanctoest::consultaItens($filial, $lancamento, $organizacao);
            if ($objItens) {
                /** @var $item Itlanctoest */
                foreach($objItens as $item) {
                    AtualizadorDeEstoque::atualizarEstoque($objLancto, $item);
                }
            } else {
                AtualizadorDeEstoque::atualizarEstoque($objLancto);
            }

            $connection->flush();
            $connection->commit();
        } catch(\Exception $e){
            TratamentoDeMensagens::exibirMensagens($e, true);
        }
    }
} 