<?php
namespace Geral\Logic\Login;

use \Geral\Entities\Filial;
use \Geral\Entities\Filioper;
use \Geral\Entities\Operador;

/**
 *
 */
class AlteraFilial
{

    private $nmoperador;
    private $senha;
    private $cdfilial;
    private $nmorganizacao;

    /** @var $organizacao \Geral\Entities\Organizacao */
    private $organizacao;
    /** @var $filial \Geral\Entities\Filial*/
    private $filial;
    /** @var $operador Operador */
    private $operador;


    /**
     * Construtor.
     *
     * @param $nmoperador
     * @param $senha
     * @param $cdfilial
     * @param $nmorganizacao
     */
    public function __construct($nmoperador, $cdfilial, $nmorganizacao)
    {
        $this->nmoperador    = $nmoperador;
        $this->cdfilial      = $cdfilial;
        $this->nmorganizacao = $nmorganizacao;
    }

    /**
     * Inicializa os objetos de orgnaizacao, filial e operador, alem de validar os dados do usuario.
     *
     * @throws \Exception
     * @return bool
     */
    public function valida()
    {
        try {
            $this->organizacao = $this->obterOrganizacao($this->nmorganizacao);
            $this->filial      = $this->obterFilial($this->cdfilial);
            $this->operador    = $this->obterOperador($this->nmoperador, $this->senha);
        } catch (\Exception $e) {
            throw $e;
        }

        return true;
    }

    /**
     * Retorna o objeto de operador.
     *
     * @return \Geral\Entities\Operador
     */
    public function getOperador()
    {
        return $this->operador;
    }

    /**
     * Retorna o objeto de filial.
     *
     * @return \Geral\Entities\Filial
     */
    public function getFilial()
    {
        return $this->filial;
    }

    /**
     * Retorna o objeto de organizacao.
     *
     * @return \Geral\Entities\Organizacao
     */
    public function getOrganizacao()
    {
        return $this->organizacao;
    }

    /**
     * Retorna o objeto de organizacao a partir do nome da organizacao ou do nrorg. Caso este nao exista, o metodo lanca
     * excessao.
     *
     * @param $nmorganizacao
     * @throws Exceptions\OrganizacaoNaoEncontrada
     * @return \Geral\Entities\Organizacao
     */
    private function obterOrganizacao($nmorganizacao)
    {
        $organizacao = \Geral\Entities\Organizacao::findOneByNmacesso($nmorganizacao);
        if (($organizacao instanceof \Geral\Entities\Organizacao) === false) {
            if (is_numeric($nmorganizacao)) {
                $organizacao = \Geral\Entities\Organizacao::findOneByNrorg($nmorganizacao);
            }
        }

        if (($organizacao instanceof \Geral\Entities\Organizacao) === false) {
            throw new \Geral\Logic\Login\Exceptions\OrganizacaoNaoEncontrada();
        }

        return $organizacao;
    }

    /**
     * Retorna o objeto de filial a partir do codigo da filial. Caso nao encontre uma filial, retorna uma mensagem de
     * erro.
     *
     * @param $cdfilial
     * @throws Exceptions\FilialNaoEncontrada
     * @return Filial
     */
    private function obterFilial($cdfilial)
    {
        $filial = Filial::findOneByCdfilialAndNrorg($cdfilial, $this->organizacao->getNrorg());

        if (($filial instanceof \Geral\Entities\Filial) === false) {
            throw new \Geral\Logic\Login\Exceptions\FilialNaoEncontrada();
        }

        return $filial;
    }

    /**
     * Retorna o objeto de operador a partir do codigo do operador. Caso nao haja nenhum operador cadastrado, uma
     * excecao deve ser lancada.
     *
     * @param $nmoperador
     * @param $senha
     * @return string
     */
    private function obterOperador($nmoperador)
    {
        $operador = Operador::findOneByNmacessoAndNrorg($nmoperador, $this->organizacao->getNrorg());
        if (($operador instanceof Operador) === false) {
            $nmoperador = str_pad($nmoperador, 12, '0', STR_PAD_LEFT);
            if(is_numeric($nmoperador)) {
                $operador = Operador::findOneByCdoperadorAndNrorg($nmoperador, $this->organizacao->getNrorg());
            }
        }

        try {
            $this->validaOperador($operador);
        } catch (\Exception $e) {
            throw $e;
        }

        return $operador;
    }

    /**
     * Valida o cadastro do operador utilizado no login.
     *
     * @param Operador $operador
     * @param $senha
     *
     * @throws Exceptions\OperadorForaDaFilial
     * @throws Exceptions\OperadorNaoEncontrado
     * @throws Exceptions\OperadorSenhaInvalida
     */
    private function validaOperador($operador)
    {
        if (($operador instanceof Operador) === false) {
            throw new \Geral\Logic\Login\Exceptions\OperadorNaoEncontrado();
        }

        $filioper = Filioper::findOneByCdfilialAndCdoperadorAndNrorg(
            $this->filial->getCdfilial(),
            $operador->getCdoperador(),
            $this->organizacao->getNrorg()
        );

        if (($filioper instanceof Filioper) === false) {
            throw new \Geral\Logic\Login\Exceptions\OperadorForaDaFilial();
        }

    }

}
